unit Unit2;

interface

uses
{$IFNDEF LINUX}
  Windows, Messages,
{$ENDIF}
{$IFDEF CLX}
  QGraphics, QControls, QForms, QDialogs, QExtCtrls, QStdCtrls, QComCtrls,
{$ELSE}
  Graphics, Controls, Forms, Dialogs, ExtCtrls, StdCtrls, ComCtrls,
{$ENDIF}
  SysUtils, Classes, VclTee.TeeThemes, VclTee.TeeProcs, VclTee.TeEngine, VclTee.Chart, VclTee.TeeComma, VclTee.TeCanvas,
  VclTee.TeeOpenGL, VclTee.TeeEditPro, VclTee.Series, VclTee.TeePoin3, VclTee.TeeSurfa, VclTee.TeeTriSurface, VclTee.TeeTools, VclTee.TeeGDIPlus,
  VclTee.TeeLisB;

type
  TForm1 = class(TForm)
    TheChart: TChart;
    Button1: TButton;
    ChartListBox1: TChartListBox;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure SetCallout(AIndex: integer);
    procedure TheChartMouseMove(Sender: TObject; Shift: TShiftState; X, Y: integer);
  private
    { Private declarations }
    NearestPointTool: TNearestTool;
    NearestPointAnnotation: TAnnotationTool;
    CalloutPointIndex: integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const
  L_Zero = 1E-2;
  A_Zero = 1E-4;
  TargetPlaneAngle = 5 * pi / 180;
  ChartMargin = 10;
  AxisMinMaxOffset = 10;
  TitleFontSize = 12;
  AxisTitleFontSize = 9;
  AxisLabelFontSize = 8;
  FrameWidth = 2;
  AxisWidth = 2;
  PointerSize = 4;
  ChartAxisLabelFormat: string = '#,##0.##';

procedure Chart_Theme(var aChart: TChart);
begin
  with TDefaultTheme.Create(aChart) do
    try
      Apply;
    finally
      Free;
    end;
end;

procedure ChartAxes_Characteristics_3D(var aChart: TChart);
var
  i: integer;
begin
  with aChart do
    begin
      View3D := true;
      Chart3DPercent := 100;
      View3DOptions.Orthogonal := false;
      View3DOptions.Zoom := 75;
      View3DOptions.Rotation := 315;
      View3DOptions.Elevation := 345;

      BevelInner := bvNone;
      BevelOuter := bvNone;
      Color := clWhite;
      Gradient.Visible := false;
      Walls.Visible := true;
      Shadow.Visible := false;

      View3DWalls := true;
      Walls.Bottom.AutoHide := true;
      Walls.Left.AutoHide := true;
      Walls.Right.AutoHide := true;
      Walls.Back.AutoHide := true;
      Walls.Bottom.Color := clBtnFace;
      Walls.Left.Color := clBtnFace;
      Walls.Back.Color := clBtnFace;
      Walls.Right.Color := clBtnFace;

      MarginUnits := muPixels;
      MarginTop := ChartMargin;
      MarginBottom := ChartMargin;
      MarginLeft := ChartMargin;
      MarginRight := ChartMargin;

      Title.Visible := false;
      Foot.Visible := true;
      Foot.Font.DefaultFont := false;
      Foot.Font.Color := clBlack;
      Foot.Font.Size := TitleFontSize;
      Foot.Font.Style := [fsBold];

      Legend.Color := clWhite;
      Legend.Shadow.Visible := false;
      Legend.CustomPosition := true;
      Legend.LegendStyle := lsSeries;
      Legend.FontSeriesColor := true;
      Legend.Font.DefaultFont := false;
      Legend.Font.Size := AxisLabelFontSize;
      Legend.Font.Style := [];

      Axes.FastCalc := true;
      for i := 0 to Axes.Count - 1 do
        begin
          with Axes[i] do
            begin
              PositionUnits := muPixels;
              Axis.Width := AxisWidth;
              Axis.Color := clBlack;
              Automatic := true;
              RoundFirstLabel := true;
              MinimumRound := true;
              MaximumRound := true;
              Title.Visible := true;
              Title.Font.DefaultFont := false;
              Title.Font.Size := AxisTitleFontSize;
              Title.Font.Style := [];
              Labels := true;
              LabelsSeparation := 1;
              LabelsFont.DefaultFont := false;
              LabelsFont.Size := AxisLabelFontSize;
              LabelsFont.Style := [];
              MinorTickCount := 3;
              AxisValuesFormat := ChartAxisLabelFormat;
            end;
        end;
      Frame.Visible := true;
      Frame.Style := LeftAxis.Axis.Style;
      Frame.Width := FrameWidth;
      Frame.Color := LeftAxis.Axis.Color;
    end;
end;

procedure Initialize_3DChart(var aChart: TChart);
begin
  Chart_Theme(aChart);
  ChartAxes_Characteristics_3D(aChart);
  with aChart do
    begin
      SeriesList.Clear;
      CustomAxes.Clear;
      if SeriesCount > 0 then
        FreeAllSeries();
    end;
end;

procedure Chart_Annotation(var aAnnotation: TAnnotationTool);
begin
  with aAnnotation do
    begin
      Position := ppLeftTop;
      with Shape do
        begin
          AutoSize := true;
          Color := clCream;
          Gradient.Visible := false;
          Shadow.Visible := false;
          Frame.Width := FrameWidth;
          Frame.Color := clBlack;
          Font.Color := clBlack;
          Font.Height := -12;
          Font.Style := [];
        end;
      with Callout do
        begin
          Visible := true;
          Style := psCircle;
          HorizSize := Round(1.5 * PointerSize);
          VertSize := Round(1.5 * PointerSize);
          Brush.Color := 33023;
          Pen.Color := clLime;
          Pen.Width := FrameWidth;
          Arrow.Visible := true;
          Arrow.Style := PsDash;
          Arrow.Width := Pen.Width;
          Arrow.Color := Pen.Color;
        end;
    end;
end;

procedure Plot3D_ChartAxes(var aChart: TChart);
begin
  with aChart do
    begin
      with Axes.Bottom do
        begin
          Visible := true;
          Title.Caption := 'EastAxis (m)';
          Inverted := false;
        end;
      with Axes.Depth do
        begin
          Visible := true;
          Title.Caption := 'NorthAxis (m)';
          Inverted := false;
        end;
      with Axes.Left do
        begin
          Visible := true;
          Title.Caption := 'TVDAxis (m)';
          Inverted := true;
        end;
    end;
end;

procedure Plot3D_AddPointSeries(var aChart: TChart; aSeriesTitle: string; aSeriesColor: TColor);
begin
  with aChart.AddSeries(TPoint3DSeries) as TPoint3DSeries do
    begin
      Title := aSeriesTitle;
      XValues.Order := loNone;
      YValues.Order := loNone;
      ZValues.Order := loNone;
      TreatNulls := tnDontPaint;
      LinePen.Visible := false;
      LinePen.Style := psSolid;
      LinePen.Width := 2;
      SeriesColor := aSeriesColor;
      LinePen.Color := SeriesColor;
      Pointer.Visible := true;
      Pointer.Style := psCircle;
      Pointer.Size := PointerSize;
      Pointer.Color := aSeriesColor;
      Pointer.Brush.Color := aSeriesColor;
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Initialize_3DChart(TheChart);
  TheChart.Legend.Visible := false;
  if NearestPointTool = nil then
    begin
      NearestPointTool := TNearestTool.Create(Self);
      TheChart.Tools.Add(NearestPointTool);
    end;
  if NearestPointAnnotation = nil then
    begin
      NearestPointAnnotation := TAnnotationTool.Create(Self);
      TheChart.Tools.Add(NearestPointAnnotation);
    end;
  Chart_Annotation(NearestPointAnnotation);

  Plot3D_ChartAxes(TheChart);
  Plot3D_AddPointSeries(TheChart, 'Test', clBlue);
  with TheChart.Series[TheChart.SeriesCount - 1] as TPoint3DSeries do
    begin
      ColorEachPoint := true;
      FillSampleValues(25);
      Pointer.VertSize := 8;
      Pointer.HorizSize := 8;
      Pointer.Style := psCircle;
      DepthSize := 0;
      Active := true;
    end;

  ChartListBox1.Chart := TheChart;
  ChartListBox1.SelectedSeries := TheChart[0];
end;

procedure TForm1.SetCallout(AIndex: integer);
begin
  with ChartListBox1.SelectedSeries as TPoint3DSeries do
    begin
      NearestPointAnnotation.Text := 'Sample';
      with NearestPointAnnotation.Callout do
        begin
          Draw3D := true;
          XPosition := CalcXPos(AIndex);
          YPosition := CalcYPos(AIndex);
          ZPosition := CalcZPos(AIndex);
        end;
    end;
end;

procedure TForm1.TheChartMouseMove(Sender: TObject; Shift: TShiftState; X, Y: integer);
begin
  inherited;
  with ChartListBox1 do
    begin
      if NearestPointTool.Active and (SelectedSeries <> nil) and SelectedSeries.Active then
        begin
          CalloutPointIndex := NearestPointTool.GetNearestPoint(SelectedSeries, X, Y);
          if CalloutPointIndex <> -1 then
            SetCallout(CalloutPointIndex);
        end;
    end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  close;
end;

end.
